# 11.2 GDBを使用してデバッグする
プログラムを開発するにあたって開発者は度々デバッグコードを書く必要があります。Go言語は、PHPやPythonといった動的な言語のようにコンパイラを必要とせず修正を行うだけで直接出力し、動的に実行環境下でデータを出力できるわけではありません。当然Go言語もPrintlnのようにデータを出力することでデバッグすることはできますが、毎回再コンパイルする必要があります。これは非常に面倒くさいことです。Pythonではpdb/ipdbのようなツールによってデバッグを行うことができますし、Javascriptにも似たようなツールがあります。これらのツールはどれも動的に変数情報を表示させることや、ステップ実行ができます。我々はGDBを使ってデバッグすることができます。ではこの節ではどのようにしてGDBによってGoプログラムをデバッグするのかご紹介しましょう。

## GDBデバッグの簡単な紹介
GDBはFSF(フリーソフトウェア財団)が配布している強力なUNIXシステムのプログラムデバッグツールです。GDBを使って以下のようなことができます：

1. プログラムを起動して、開発者が定義した要求にしたがってプログラムを実行できます。
2. デバッグされるプログラムは開発者の設定したブレークポイントで止めることができます。（ブレークポイントは条件式にすることができます。）
3. プログラムが停止した時、この時のプログラムで発生している事柄を検査することができます。
4. 動的に現在のプログラムの実行環境を変更することができます。

現在GoプログラムのデバッグをサポートしているGDBのバージョンは7.1以上です。

Goプログラムをコンパイルするときは以下のいくつかに注意してください

1. パラメータは-ldflags "-s"は、debugの情報の出力を省略します。
2. -gcflags "-N -l" パラメータではGoの内部で行われるいくつかの最適化を無視できます。集成体型変数と関数の最適化です。これらはGDBのデバッグでは非常に困難ですので、コンパイルする時にこの２つのパラメータを追加することで最適化を避けます。

## よく使うコマンド
GDBでよく使うコマンドのいくつかは以下の通りです

- list

	`l`と省略されます。ソースコードを表示するために使用されます。デフォルトで10行のコードを表示します。後ろに表示する具体的な行をパラメータとして渡すことができます。例えば：`list 15`では10行のコードを表示し、以下のように15行目が10行のうちの中心に表示されます。

		10	        time.Sleep(2 * time.Second)
		11	        c <- i
		12	    }
		13	    close(c)
		14	}
		15	
		16	func main() {
		17	    msg := "Starting main"
		18	    fmt.Println(msg)
		19	    bus := make(chan int)

	
- break

	`b`と省略されます。ブレークポイントを設定するために用いられます。後ろにブレークポイントを置く行をパラメータとして追加します。例えば`b 10`では10行目にブレークポイントが置かれます。
	
- delete
	`d`と省略されます。ブレークポイントを削除するために用いられます。後ろにブレークポイントの番号がつきます。この番号は`info breakpoints`によって対応する設定されたブレークポイントの番号を取得できます。以下では設定されたブレークポイントの番号を表示します。

		Num     Type           Disp Enb Address            What
		2       breakpoint     keep y   0x0000000000400dc3 in main.main at /home/xiemengjun/gdb.go:23
		breakpoint already hit 1 time

- backtrace
	
	`bt`と省略されます。以下のように実行しているコードの過程を出力するために用いられます：

		#0  main.main () at /home/xiemengjun/gdb.go:23
		#1  0x000000000040d61e in runtime.main () at /home/xiemengjun/go/src/pkg/runtime/proc.c:244
		#2  0x000000000040d6c1 in schedunlock () at /home/xiemengjun/go/src/pkg/runtime/proc.c:267
		#3  0x0000000000000000 in ?? ()
- info

	infoコマンドは情報を表示します。後ろにいくつかのパラメータがあります。よく使われるものは以下のいくつかです：
		
	- `info locals`

		現在実行しているプログラムの変数の値を表示します。
	- `info breakpoints`

		現在設定しているブレークポイントのリストを表示します。
	- `info goroutines`

		現在実行しているgoroutineのリストを表示します。以下のコードが示すとおり*がついているものは現在実行しているものです。

			* 1  running runtime.gosched
			* 2  syscall runtime.entersyscall
			  3  waiting runtime.gosched
			  4 runnable runtime.gosched
- print

	`p`と省略されます。変数またはその他の情報を表示するのに用いられます。後ろに出力する必要のある変数名が追加されます。当然とても使いやすい関数$len()と$cap()もあります。現在のstring、slicesまたはmapsの長さと容量を返すのに使われます。

- whatis 
	
	現在の変数の型を表示するのに用いられます。後ろに変数名がつきます。たとえば`whatis msg`では以下のように表示されます：

		type = struct string
- next

	`n`と省略されます。ステップ実行に使われます。次のステップに進みます。ブレークポイントがあれば`n`を入力することで次のステップまで続けて実行することができます。
- coutinue

	`c`と省略されます。現在のブレークポイントから抜けます。後ろにパラメータをつけることで、何回かのブレークポイントを飛び越えることができます。

- set variable

	このコマンドは実行中の変数の値を変更するのに用いられます。フォーマットは以下のとおり： `set variable <var>=<value>`

## デバッグ過程
以下のコードによってどのようにGDBを使ってGoプログラムをデバッグするのかデモを行います。以下はデモコードです：

	package main

	import (
		"fmt"
		"time"
	)

	func counting(c chan<- int) {
		for i := 0; i < 10; i++ {
			time.Sleep(2 * time.Second)
			c <- i
		}
		close(c)
	}

	func main() {
		msg := "Starting main"
		fmt.Println(msg)
		bus := make(chan int)
		msg = "starting a gofunc"
		go counting(bus)
		for count := range bus {
			fmt.Println("count:", count)
		}
	}

ファイルをコンパイルして実行可能ファイルgdbfileを生成します:

	go build -gcflags "-N -l" gdbfile.go

gdbコマンドによってデバッグを起動します：

	gdb gdbfile
	
起動したらまずこのプログラムが実行できるか見てみましょう。`run`コマンドを入力してエンターキーを押すとプログラムが実行されます。プログラムが正常であれば、プログラムは以下のように出力します。コマンドラインで直接プログラムを実行したのと同じです：

	(gdb) run
	Starting program: /home/xiemengjun/gdbfile 
	Starting main
	count: 0
	count: 1
	count: 2
	count: 3
	count: 4
	count: 5
	count: 6
	count: 7
	count: 8
	count: 9
	[LWP 2771 exited]
	[Inferior 1 (process 2771) exited normally]	
よし、プログラムをどのようにして起動するかわかりました。次にプログラムにブレークポイントを設定します：

	(gdb) b 23
	Breakpoint 1 at 0x400d8d: file /home/xiemengjun/gdbfile.go, line 23.
	(gdb) run
	Starting program: /home/xiemengjun/gdbfile 
	Starting main
	[New LWP 3284]
	[Switching to LWP 3284]

	Breakpoint 1, main.main () at /home/xiemengjun/gdbfile.go:23
	23	        fmt.Println("count:", count)

上の例では`b 23`で23行目にブレークポイントを設定しました。その後`run`を入力するとプログラムが開始します。現在プログラムは前に設定されたブレークポイントで停止しています。ブレークポイントに対応するコンテキストのソースコードを知るためには、`list`と入力することでソースコードが現在停止している行の前の5行から表示させることができます：

	(gdb) list
	18	    fmt.Println(msg)
	19	    bus := make(chan int)
	20	    msg = "starting a gofunc"
	21	    go counting(bus)
	22	    for count := range bus {
	23	        fmt.Println("count:", count)
	24	    }
	25	}

GDBが実行している現在のプログラムの環境ではいくつかの便利なデバッグ情報を持っています。対応する変数を出力するだけで、対応する変数の型と値を確認することができます：

	(gdb) info locals
	count = 0
	bus = 0xf840001a50
	(gdb) p count
	$1 = 0
	(gdb) p bus
	$2 = (chan int) 0xf840001a50
	(gdb) whatis bus
	type = chan int

次にこのプログラムを継続して実行させ続けなければなりません。以下のコマンドをご覧ください

	(gdb) c
	Continuing.
	count: 0
	[New LWP 3303]
	[Switching to LWP 3303]

	Breakpoint 1, main.main () at /home/xiemengjun/gdbfile.go:23
	23 fmt.Println("count:", count)
	(gdb) c
	Continuing.
	count: 1
	[Switching to LWP 3302]

	Breakpoint 1, main.main () at /home/xiemengjun/gdbfile.go:23
	23 fmt.Println("count:", count)

毎回`c`を入力する度に一回のコードが実行されます。次のforループにジャンプして、続けて対応する情報を出力します。

現在コンテキストの関連する変数の情報を変えたいとします。いくつかのプロセスを飛び越えて、続けて次のステップを実行し、修正を行った後に欲しい結果を得ます：

	(gdb) info locals
	count = 2
	bus = 0xf840001a50
	(gdb) set variable count=9
	(gdb) info locals
	count = 9
	bus = 0xf840001a50
	(gdb) c
	Continuing.
	count: 9
	[Switching to LWP 3302]

	Breakpoint 1, main.main () at /home/xiemengjun/gdbfile.go:23
	23 fmt.Println("count:", count)		
	
最後に少しだけ考えてみましょう。前のプログラムの実行の全過程ではいくつのgorutineが作成されたでしょうか。各goroutineは何をやっているのでしょうか：

	(gdb) info goroutines
	* 1 running runtime.gosched
	* 2 syscall runtime.entersyscall 
	3 waiting runtime.gosched 
	4 runnable runtime.gosched
	(gdb) goroutine 1 bt
	#0 0x000000000040e33b in runtime.gosched () at /home/xiemengjun/go/src/pkg/runtime/proc.c:927
	#1 0x0000000000403091 in runtime.chanrecv (c=void, ep=void, selected=void, received=void)
	at /home/xiemengjun/go/src/pkg/runtime/chan.c:327
	#2 0x000000000040316f in runtime.chanrecv2 (t=void, c=void)
	at /home/xiemengjun/go/src/pkg/runtime/chan.c:420
	#3 0x0000000000400d6f in main.main () at /home/xiemengjun/gdbfile.go:22
	#4 0x000000000040d0c7 in runtime.main () at /home/xiemengjun/go/src/pkg/runtime/proc.c:244
	#5 0x000000000040d16a in schedunlock () at /home/xiemengjun/go/src/pkg/runtime/proc.c:267
	#6 0x0000000000000000 in ?? ()

goroutinesのコマンドを確認することでgoroutineの内部がどのように実行されているのか詳しく理解することができます。各関数のコールされる順番はすでにはっきり表示されています。

## まとめ
この章ではGDBデバッグにおけるGoプログラムの基本コマンドのいくつかをご紹介しました。`run`、`print`、`info`、`set variable`、`continue`、`list`、`break`といったよく使われるデバッグコマンドを含め、上のデモで行ったように、読者はすでにGoプログラムに対してGDBを使ったデバッグを基本的に理解したものと信じています。もしより多くのデバッグテクニックを知りたければオフィシャルのページのGDBデバッグの項目をご参照ください。
	
## links
   * [目次](<preface.md>)
   * 前へ: [エラー処理](<11.1.md>)
   * 次へ: [Goでどのようにテストを書くか](<11.3.md>)
