# 5.6 NoSQL database

A NoSQL database provides a mechanism for the storage and retrieval of data that uses looser consistency models than typical relational databases in order to achieve horizontal scaling and higher availability. Some authors refer to them as "Not only SQL" to emphasize that some NoSQL systems do allow SQL-like query languages to be used.

As the C language of the 21st century, Go has good support for NoSQL databases, including the popular redis, mongoDB, Cassandra and Membase NoSQL databases.

## redis

redis is a key-value storage system like Memcached, that supports the string, list, set and zset(ordered set) value types.

There are some Go database drivers for redis:

- [https://github.com/alphazero/Go-Redis](https://github.com/alphazero/Go-Redis)
- [http://code.google.com/p/tideland-rdc/](http://code.google.com/p/tideland-rdc/)
- [https://github.com/simonz05/godis](https://github.com/simonz05/godis)
- [https://github.com/hoisie/redis.go](https://github.com/hoisie/redis.go)

I forked the last of these packages, fixed some bugs, and used it in my short URL service (2 million PV every day).

- [https://github.com/astaxie/goredis](https://github.com/astaxie/goredis)

Let's see how to use the driver that I forked to operate on a database:

	package main

	import (
		"github.com/astaxie/goredis"
		"fmt"
	)

	func main() {
		var client goredis.Client
		
		// Set the default port in Redis
		client.Addr = "127.0.0.1:6379"
		
		// string manipulation
		client.Set("a", []byte("hello"))
		val, _ := client.Get("a")
		fmt.Println(string(val))
		client.Del("a")

		// list operation
		vals := []string{"a", "b", "c", "d", "e"}
		for _, v := range vals {
			client.Rpush("l", []byte(v))
		}
		dbvals,_ := client.Lrange("l", 0, 4)
		for i, v := range dbvals {
			println(i,":",string(v))
		}
		client.Del("l")
	}

We can see that it's quite easy to operate redis in Go, and it has high performance. Its client commands are almost the same as redis' built-in commands.

## mongoDB

mongoDB (from "humongous") is an open source document-oriented database system developed and supported by 10gen. It is part of the NoSQL family of database systems. Instead of storing data in tables as is done in a "classical" relational database, MongoDB stores structured data as JSON-like documents with dynamic schemas (MongoDB calls the format BSON), making the integration of data in certain types of applications easier and faster.

![](images/5.6.mongodb.png?raw=true)

Figure 5.1 MongoDB compared to Mysql

The best driver for mongoDB is called `mgo`, and it is possible that it will be included in the standard library in the future.

Here is the example:

	package main
	
	import (
	    "fmt"
	    "labix.org/v2/mgo"
	    "labix.org/v2/mgo/bson"
	)
	
	type Person struct {
	    Name string
	    Phone string
	}
	
	func main() {
	    session, err := mgo.Dial("server1.example.com,server2.example.com")
	    if err != nil {
	        panic(err)
	    }
	    defer session.Close()
	
	    session.SetMode(mgo.Monotonic, true)
	
	    c := session.DB("test").C("people")
	    err = c.Insert(&Person{"Ale", "+55 53 8116 9639"},
	        &Person{"Cla", "+55 53 8402 8510"})
	    if err != nil {
	        panic(err)
	    }
	
	    result := Person{}
	    err = c.Find(bson.M{"name": "Ale"}).One(&result)
	    if err != nil {
	        panic(err)
	    }
	
	    fmt.Println("Phone:", result.Phone)
	}

We can see that there are no big differences when it comes to operating on mgo or beedb databases; they are both based on structs. This is the Go way of doing things.

## Links

- [Directory](preface.md)
- Previous section: [Develop ORM based on beedb](05.5.md)
- Next section: [Summary](05.7.md)
